package client;


import file.sr.TesteReceiver;
import file.sr.TesteSender;
import gui.ChatWindow;
import gui.GroupChatWindow;

import java.io.IOException;
import java.net.DatagramSocket;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.DefaultListModel;
import javax.swing.JOptionPane;

import server.User;
import voip.Voice;

import net.ConnectionInfo;
import net.TimeoutLimitException;

public class ChatManager implements ChatListener{
	private Hashtable<Byte, GroupChatWindow> tabelaDeJanelasEntrada; //Mapeia os ids de entrada
	private Hashtable<GroupChatWindow, Byte> tabelaControle;
	private Hashtable<GroupChatWindow, Hashtable<ConnectionInfo, Byte>> tabelaDeJanelasSaida;  //Mapeia os ids de saida
	private Hashtable<ConnectionInfo, Byte> controleJanelas;
	private String login;
	private FachadaClient fachada;
	private DefaultListModel modeloAmigos;
	private Client client;
	private byte nextId;
	private boolean emChamada;
	private boolean emTransferencia;
	private Voice voz;
		
	public ChatManager(Client client, FachadaClient fachada, String login, DefaultListModel modeloAmigos){
		this.login = login;
		this.fachada = fachada;
		this.modeloAmigos = modeloAmigos;
		this.emChamada = false;
		this.client = client;
		this.nextId = 0;
		tabelaDeJanelasEntrada = new Hashtable<Byte, GroupChatWindow>();
		tabelaDeJanelasSaida = new Hashtable<GroupChatWindow, Hashtable<ConnectionInfo, Byte>>();
		tabelaControle = new Hashtable<GroupChatWindow, Byte>();
		controleJanelas = new Hashtable<ConnectionInfo, Byte>();
	}
	public void destroyJanelas(){
		Enumeration<GroupChatWindow> janelas = tabelaControle.keys();
		while(janelas.hasMoreElements()){
			janelas.nextElement().dispose();
		}
		tabelaDeJanelasEntrada.clear();
		tabelaControle.clear();
		tabelaDeJanelasSaida.clear();
		controleJanelas.clear();
	}
	public void sendMsg(String msg,  GroupChatWindow janela, Hashtable<ConnectionInfo, String> tabela) throws  GroupException{
		boolean deuErro = false;
		Hashtable<ConnectionInfo, Byte> participantes = tabelaDeJanelasSaida
				.get(janela);
		Enumeration<ConnectionInfo> chaves = null;
		if(participantes == null){
			participantes = new Hashtable<ConnectionInfo, Byte>();
			chaves = tabela.keys();
		}else{
			chaves = participantes.keys();
		}
		try{
		while (chaves.hasMoreElements()) {
			ConnectionInfo ci = chaves.nextElement();
			Byte id = participantes.get(ci);
			if (id != null) { // Se j possuia o id da janela de destino
				System.out.println("MANDOU PARA O ID: "+id);
					client.sendMsg(msg, ci, id);
				
			} else { // se no
				System.out.println("CHAT MANAGER REQUERIU ID");
				byte idRetorno = client.requestId(ci, nextId); // Pede id
				tabelaDeJanelasEntrada.put(nextId, janela);
				tabelaControle.put(janela, nextId);
				nextId++;
				participantes.put(ci, idRetorno);
				tabelaDeJanelasSaida.put(janela, participantes);
				client.sendMsg(msg, ci, idRetorno); // envia msg
			}
		}
		}catch(TimeoutLimitException e){
			deuErro = true;
		}
		if(deuErro){
			throw new GroupException("No foi possvel enviar a mensagem a todos os destinatrios");
		}
	}
	public void realizarChamada(ConnectionInfo ci, DatagramSocket voiceSock) throws LinhaOcupadaException, TimeoutLimitException, UsuarioNegouException, SocketException, InterruptedException{
		if(emChamada){
			throw new LinhaOcupadaException("S  possvel realizar uma chamada por vez");
		}else{
			client.chamadaDeVoz(ci);
			emChamada = true;
			voz = new Voice(ci, voiceSock);
			voz.comeco();
		}
	}
	public void mandarArquivo(ConnectionInfo ci, String nome) throws IOException, TimeoutLimitException, UsuarioNegouException, InterruptedException{
		TesteSender sender = new TesteSender(nome, ci.getIp().getHostAddress());
		client.mandarArquivo(ci, nome);
		
		sender.start();
		
	}
	public void terminarChamda() throws InterruptedException{
		if(emChamada){
			voz.termino();
		}
	}
	public void convidarParaConversa(GroupChatWindow janela,Hashtable<ConnectionInfo, String> tabela, ConnectionInfo ci) throws TimeoutLimitException, IOException{
		Hashtable<ConnectionInfo, Byte> participantes = tabelaDeJanelasSaida.get(janela);
		Enumeration<ConnectionInfo> chaves = null;
		if(controleJanelas.containsKey(ci)){
			controleJanelas.remove(ci); // libera chat para usuario simples
		}
		if(participantes == null){ // Se tiver pedido antes de fazer conexao...
			participantes = new Hashtable<ConnectionInfo, Byte>();
			chaves = tabela.keys();
			while(chaves.hasMoreElements()){
				ConnectionInfo temp = chaves.nextElement();
				System.out.println("CHAT MANAGER REQUERIU ID");
				byte idRetorno = client.requestId(temp, nextId); // Pede id
				tabelaDeJanelasEntrada.put(nextId, janela);
				tabelaControle.put(janela, nextId);
				nextId++;
				participantes.put(temp, idRetorno);
				tabelaDeJanelasSaida.put(janela, participantes);
			}
		}
		ArrayList<ChatId> contatos = new ArrayList<ChatId>();
		chaves = participantes.keys();
		while(chaves.hasMoreElements()){
			ConnectionInfo coisa = chaves.nextElement();
			Byte loucura = participantes.get(coisa);
			contatos.add(new ChatId(coisa, loucura));
		}
		byte id = tabelaControle.get(janela);
		client.convidaParaGroup(ci, contatos, id);
		
	}
	
	
	
	@Override
	public void msgRecieved(String msg, ConnectionInfo ci, byte idDestino) {
		GroupChatWindow janela = tabelaDeJanelasEntrada.get(new Byte(idDestino));
		System.out.println("MANAGER RECEBEU MSG");
		if(janela != null){
			janela.msgRecieved(msg, ci, null);//Arrumar o null
		}
	}

	@Override
	public void callRecieved(ConnectionInfo ci){
		try{
		if(emChamada){
			client.responderChamdaDeVoz(ci,(byte) 0);
	
		}else{
			String login = this.getLogin(ci);
			int coisa = JOptionPane.showConfirmDialog(null,
		                "Deseja conversar com "+login+"?", "Chat de voz", JOptionPane.WARNING_MESSAGE);
			  if(coisa == 0){
				  client.responderChamdaDeVoz(ci, (byte)1);
				  voz = new Voice(ci, fachada.getSockVoz());
				  voz.comeco();
				  this.emChamada = true;
			  }else{
				  client.responderChamdaDeVoz(ci, (byte)0);
				  System.out.println("NEGOU");
			  }
			 
		}
		}catch(TimeoutLimitException e){
			//alallalalalallal
		} catch (SocketException e) {

			e.printStackTrace();
		}
	}

	@Override
	public void transferRecieved(FileData data, ConnectionInfo ci) {
		System.out.println("NOME DO ARQUIVO: "+data.getNome()+"TAMANHO: "+data.getTamanho());
		if(emTransferencia){
			
		}else{
			try {
				TesteReceiver rcv = new TesteReceiver(data.getNome(), ci.getIp().getHostAddress());
				rcv.run();
			} catch (UnknownHostException e) {
				e.printStackTrace();
			}
		}
		
	}

	@Override
	public void conviteDeConversaRecebido(ArrayList<ChatId> participantes, byte idOrigem, ConnectionInfo origem){
		Hashtable<ConnectionInfo, String> tabelaDeParticipantes = new Hashtable<ConnectionInfo, String>();
		for(ChatId participante : participantes){
			tabelaDeParticipantes.put(participante.getConnectionInfo(), this.getLogin(participante.getConnectionInfo()));
		}
		tabelaDeParticipantes.put(origem, this.getLogin(origem));
		GroupChatWindow janela = new GroupChatWindow(this.login, this.fachada, tabelaDeParticipantes, modeloAmigos);
		janela.setVisible(true);
		byte back = nextId;
		this.tabelaDeJanelasEntrada.put(nextId, janela);
		this.tabelaControle.put(janela, nextId);
		nextId++;
		Hashtable<ConnectionInfo, Byte> saida = this.tabelaDeJanelasSaida.get(janela);
		saida = new Hashtable<ConnectionInfo, Byte>();
		
		for(ChatId participante : participantes){
			saida.put(participante.getConnectionInfo(), participante.getId());
			try {
				client.entrarNoChat(participante.getConnectionInfo(), participante.getId(), back);
			} catch (TimeoutLimitException e) {
				e.printStackTrace();
			}
		}
		saida.put(origem, idOrigem);
		tabelaDeJanelasSaida.put(janela, saida);
		try {
			client.entrarNoChat(origem, idOrigem, back);
		} catch (TimeoutLimitException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	public void abrirJanela(String login, ConnectionInfo ci){
		if(!controleJanelas.containsKey(ci)){
			controleJanelas.put(ci, nextId);
			Hashtable<ConnectionInfo, String> temp = new Hashtable<ConnectionInfo, String>();
			temp.put(ci, login);
			GroupChatWindow janela = new GroupChatWindow(this.login,this.fachada, temp, this.modeloAmigos);
			tabelaDeJanelasEntrada.put(nextId, janela);
			tabelaControle.put(janela, nextId);
			nextId++;
			janela.setVisible(true);
		}else{
			byte id = controleJanelas.get(ci);
			tabelaDeJanelasEntrada.get(id).setVisible(true);
		}
	}

	@Override
	public void confirmouConvite(ConnectionInfo ci, ChatThread chat) {
		
	}
	@SuppressWarnings("unchecked")
	@Override
	public byte idRequestReceived(ConnectionInfo ci, byte id) {
		if (!controleJanelas.containsKey(ci)) {
			Hashtable<ConnectionInfo, String> tabelaDeParticipantes = new Hashtable<ConnectionInfo, String>();
			Enumeration<User> temp = (Enumeration<User>) modeloAmigos
					.elements();
			String login = "";
			while (temp.hasMoreElements()) {
				User coisa = temp.nextElement();
				ConnectionInfo ciCoisa = coisa.getConnectionInfo();
				if (ciCoisa != null) {
					if (ciCoisa.equals(ci)) {
						login = coisa.getLogin();
					}
				}
			}
			tabelaDeParticipantes.put(ci, login);
			GroupChatWindow janela = new GroupChatWindow(this.login,
					this.fachada, tabelaDeParticipantes, modeloAmigos);
			this.tabelaDeJanelasEntrada.put((byte) (nextId), janela);
			this.tabelaControle.put(janela, (byte) (nextId));
			Hashtable<ConnectionInfo, Byte> tabelaTemp = new Hashtable<ConnectionInfo, Byte>();
			tabelaTemp.put(ci, id);
			this.tabelaDeJanelasSaida.put(janela, tabelaTemp);
			janela.setVisible(true);
			nextId++;
			return ((byte)(this.nextId-1));
		} else {
			return controleJanelas.get(ci);
		}
	}

	@Override
	public byte idReceived(byte id) {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public void novoParticipante(ConnectionInfo origem, byte idOrigem,
			byte idDestino) {
		GroupChatWindow janela = this.tabelaDeJanelasEntrada.get(idDestino);
		Byte id = controleJanelas.get(origem);
		if(id != null){ // Entrou em mult-chat, libera a criao da mesma janela para chat individual
			controleJanelas.remove(origem); 
		}
		String login = this.getLogin(origem);
		janela.addPeer(login, origem);
		Hashtable<ConnectionInfo, Byte> participantes = this.tabelaDeJanelasSaida.get(janela);
		if(participantes == null){
			participantes = new Hashtable<ConnectionInfo, Byte>();
			participantes.put(origem, idOrigem);
			this.tabelaDeJanelasSaida.put(janela, participantes);
		}else{
			participantes.put(origem, idOrigem);
		}
	}
	@SuppressWarnings("unchecked")
	private String getLogin(ConnectionInfo ci){
		String retorno = "";
		Enumeration<User> contatos = (Enumeration<User>) modeloAmigos.elements();
		while(contatos.hasMoreElements()){
			User temp = contatos.nextElement();
			if(temp.getConnectionInfo() != null ){
				if(temp.getConnectionInfo().equals(ci)){
					retorno = temp.getLogin();
				}
			}
		}
		return retorno;
	}


}
