package client;

import gui.ChatWindow;
import gui.GroupChatWindow;

import java.io.IOException;
import java.net.DatagramSocket;
import java.net.SocketException;
import java.util.Hashtable;

import javax.swing.DefaultListModel;

import net.ConnectionInfo;
import net.TimeoutLimitException;
import server.LoginException;
import server.UnnavailableLoginException;
import server.User;

public class FachadaClient {
	private Client client;
	private ChatManager manager;
	private DatagramSocket sockVoz;
	private DatagramSocket sockArquivo;
	public FachadaClient(Client client,DatagramSocket sockVoz, DatagramSocket sockArquivo){
		this.client=client;
		this.sockVoz = sockVoz;
		this.sockArquivo = sockArquivo;
	}
	public DatagramSocket getSockVoz(){
		return this.sockVoz;
	}
	public void cadastrar(User usuario) throws IOException, TimeoutLimitException, UnnavailableLoginException, ServerException{
		client.cadastrar(usuario);
	}
	public void logar(String login, String senha) throws TimeoutLimitException, LoginException{
		client.logar(login, senha);
	}
	public void initManager(String login, DefaultListModel modeloAmigos){
		manager = new ChatManager(client, this, login, modeloAmigos);
		client.addChatListener(manager);
	}
	public void addListener(ClientGuiListener listener){
		client.addGuiListener(listener);
	}
	public void logout() throws TimeoutLimitException{
		client.logout();
	}
	public void addFriend(String login) throws LoginException, ServerException, TimeoutLimitException, IOException, UnnavailableLoginException{
		client.adicionarAmigo(login);
	}
	public void sendMsg(String msg, GroupChatWindow listener, Hashtable<ConnectionInfo, String> tabela) throws  GroupException{
		manager.sendMsg(msg, listener, tabela);
	}
	public void fecharJanelas(){
		manager.destroyJanelas();
	}
	public void convidar(ConnectionInfo ci, Hashtable<ConnectionInfo, String> tabela, GroupChatWindow janela) throws IOException, TimeoutLimitException{
		manager.convidarParaConversa(janela, tabela, ci);
	}
	public void abrirJanela(String login, ConnectionInfo ci){
		manager.abrirJanela(login, ci);
		
	}
	public void sendGroupMsg(String msg, Hashtable<ConnectionInfo, String> tabelaDeParticipantes) throws GroupException{
		client.sendGroup(msg, tabelaDeParticipantes);
	}
	public void realizarChamada(ConnectionInfo ci) throws TimeoutLimitException, SocketException, LinhaOcupadaException, UsuarioNegouException, InterruptedException{
		manager.realizarChamada(ci, this.sockVoz);
	}
	public void mandarArquivo(ConnectionInfo ci, String nome) throws IOException, TimeoutLimitException, UsuarioNegouException, InterruptedException{
		manager.mandarArquivo(ci, nome);
	}

}
